clear all
close all
addpath('mex_source/');
addpath('tools/');

% Number of transshipment locations
d_z=16;

%value of p
p=2;

% Dimension of data
d_x=8000;
d_y=8000;
disp(['Problem of dimension ' num2str(d_x) ' x ' num2str(d_y) ]);
% Simple regular 1D test
pos_x= (0: 1 / (d_x - 1): 1).';
pos_y= (0: 1 / (d_y - 1): 1).';

mu_x=exp(-(pos_x(:,1) - 0.5).^2 / 0.2^2);
mu_y=exp(-(pos_y(:,1) - 0.25).^2 / 0.1^2) + exp(-(pos_y(:,1) - 0.75).^2 / 0.1^2);


if 1  %random  test
    dim=2; 
    pos_x=rand(d_x,dim);
    pos_y=rand(d_y,dim);

    mu_x=rand(d_x,1);
    mu_y=exp(-(pos_y(:,1) - 0.25).^2 / 0.1^2) + exp(-(pos_y(:,1) - 0.75).^2 / 0.1^2);

end

% normalize data
while abs(sum(mu_x(:))-sum(mu_y(:)))>1e-15
    mu_x=mu_x/sum(mu_x(:));
    mu_y=mu_y/sum(mu_y(:));
end




compute_matrix=0; % Set to 0 for only computing the approximate distance and not the approximate transport matrix

% Start parpool if parallel toolbox available
v = ver;
[installedToolboxes{1:length(v)}] = deal(v.Name);
if all(ismember('Parallel Computing Toolbox',installedToolboxes))
    gcp;
end

% Multi-scale procedure for Wasserstein approximation
tic
[cost_app, gamma_app]=compute_Approximate_WP(pos_x,mu_x,pos_y,mu_y,p,d_z,compute_matrix);
tt_app=toc;

display(['Approximate Wp: ' num2str(cost_app.^(1/p)) ' obtained in ' num2str(tt_app) ' seconds.'])


% Exact Wasserstein
tic
[cost, gamma]=compute_WP(pos_x,mu_x,pos_y,mu_y,p);
tt=toc;
display(['True Wp: ' num2str(cost.^(1/p)) ' obtained in ' num2str(tt) ' seconds.'])


display(['Relative error: ' num2str((cost_app.^(1/p)-cost.^(1/p))/cost.^(1/p)*100) '%'])


% Plot transport matices
if compute_matrix
subplot 121
imagesc(gamma);
title('True Transport matrix')
subplot 122
imagesc(gamma_app);
title('Approximate Transport matrix')
end
