function [cost, gamma]=compute_WP(pos_x,mu_x,pos_y,mu_y,p)

% inputs:
% pos_x : dirac locations of the first density. Matrix (d_x , dim)
% mu_x  : dirac weights. Vector (d_x , 1)
% pos_y : dirac locations of the second density. Matrix (d_y , dim)
% mu_y  : dirac weights. Vector (d_y , 1)
% d_z   : number of diracs to approximate the barycenter. Positive integer
% p     : p-Wasserstin distance. Scalar >1
%

%outputs:
% cost : the approximate p-Wasserstein cost between (x,mu_x) and (y,mu_y)
% gamma: the corresponding approximation of the optimal matrix


max_iter=100;
stop_crit=1e-3;

cost=1e10;
gamma=[];
if min(mu_x(:))<0
    disp('error: mu_x should have non negative values')
    return
end

if min(mu_y(:))<0
    disp('error: mu_y should have non negative values')
    return
end

if abs(sum(mu_x(:))-sum(mu_y(:)))>1e-15
    disp('normalization: mu_x and mu_y should have the same mass')
    mu_y=mu_y*sum(mu_x(:))/sum(mu_y(:));
end


d_x=length(mu_x);


if size(pos_x,1)~=d_x
    disp('error: length of input histogram mu_x and first dimension of pos_x should be the same')
    return
end

d_y=length(mu_y);
if size(pos_y,1)~=d_y
    disp('error: length of input histogram mu_y and first dimension of pos_y should be the same')
    return
end

dim=size(pos_x,2);
if size(pos_y,2)~=dim
    disp('error: set of points in pos_x and pos_y should be of the same dimension')
    return
end




if p<1
    disp('error: p has to be greater than 1')
    return
end


C_xyd= @(d)  repmat(pos_y(:,d)',[d_x 1])-repmat(pos_x(:,d),[1 d_y]);


%update cost matrix
C = abs(C_xyd(1)).^p;
for d=2:dim
    C = C+ abs(C_xyd(d)).^p;
end
%compute barycenter between mu0 and mu1 on the grid z
[cost, gamma]=mexEMD(mu_x,mu_y,C);

